@@ -14,6 +14,9 @@
 #' @param precision The number of significant digits to include iin WKT
 #'   output.
 #' @param endian 0 for big endian or 1 for little endian.
+#' @param flavor One of "extended" (i.e., EWKB) or "iso".
+#' @param indent The number of spaces to use when indenting a formatted
+#'   version of the output. Use -1 to indicate no formatting.
 #' @inheritParams geos_segment_intersection
 #' @param hex A hexidecimal representation of well-known binary
 #' @inheritParams wk::wk_crs
@@ -40,6 +43,22 @@ geos_write_wkt <- function(geom, include_z = TRUE, precision = 16, trim = TRUE)
   )
 }
 
+#' @rdname geos_read_wkt
+#' @export
+geos_read_geojson <- function(wkt, crs = NULL) {
+  new_geos_geometry(.Call(geos_c_read_geojson, as.character(wkt)), crs = crs)
+}
+
+#' @rdname geos_read_wkt
+#' @export
+geos_write_geojson <- function(geom, indent = -1) {
+  .Call(
+    geos_c_write_geojson,
+    sanitize_geos_geometry(geom),
+    sanitize_integer_scalar(indent)
+  )
+}
+
 #' @rdname geos_read_wkt
 #' @export
 geos_read_wkb <- function(wkb, crs = NULL) {
@@ -48,14 +67,18 @@ geos_read_wkb <- function(wkb, crs = NULL) {
 
 #' @rdname geos_read_wkt
 #' @export
-geos_write_wkb <- function(geom, include_z = TRUE, include_srid = FALSE, endian = 1) {
+geos_write_wkb <- function(geom, include_z = TRUE, include_srid = FALSE, endian = 1,
+                           flavor = c("extended", "iso")) {
+  flavor <- match.arg(flavor)
+
   structure(
     .Call(
       geos_c_write_wkb,
       sanitize_geos_geometry(geom),
       sanitize_logical_scalar(include_z),
       sanitize_logical_scalar(include_srid),
-      sanitize_integer_scalar(endian)
+      sanitize_integer_scalar(endian),
+      match(flavor, c("extended", "iso"))
     ),
     class = "blob"
   )
@@ -69,13 +92,17 @@ geos_read_hex <- function(hex, crs = NULL) {
 
 #' @rdname geos_read_wkt
 #' @export
-geos_write_hex <- function(geom, include_z = TRUE, include_srid = FALSE, endian = 1) {
+geos_write_hex <- function(geom, include_z = TRUE, include_srid = FALSE, endian = 1,
+                           flavor = c("extended", "iso")) {
+  flavor <- match.arg(flavor)
+
   .Call(
     geos_c_write_hex,
     sanitize_geos_geometry(geom),
     sanitize_logical_scalar(include_z),
     sanitize_logical_scalar(include_srid),
-    sanitize_integer_scalar(endian)
+    sanitize_integer_scalar(endian),
+    match(flavor, c("extended", "iso"))
   )
 }
 
