@@ -80,6 +80,96 @@ SEXP geos_c_write_wkt(SEXP input, SEXP includeZ, SEXP precision, SEXP trim) {
   return result;
 }
 
+SEXP geos_c_read_geojson(SEXP input) {
+#if LIBGEOS_VERSION_COMPILE_INT >= LIBGEOS_VERSION_INT(3, 10, 0)
+  if (libgeos_version_int() < LIBGEOS_VERSION_INT(3, 10, 0)) {
+    ERROR_OLD_LIBGEOS("GEOSGeoJSONReader_create_r()", "3.10.0");
+  }
+
+  R_xlen_t size = Rf_xlength(input);
+  SEXP result = PROTECT(Rf_allocVector(VECSXP, size));
+
+  GEOS_INIT();
+  GEOSGeoJSONReader* reader = GEOSGeoJSONReader_create_r(handle);
+
+  GEOSGeometry* geometry;
+  for (R_xlen_t i = 0; i < size; i++) {
+    if (STRING_ELT(input, i) == NA_STRING) {
+      SET_VECTOR_ELT(result, i, R_NilValue);
+      continue;
+    }
+
+    geometry = GEOSGeoJSONReader_readGeometry_r(handle, reader, CHAR(STRING_ELT(input, i)));
+
+    // returns NULL on error
+    if (geometry == NULL) {
+      GEOSGeoJSONReader_destroy_r(handle, reader);
+      Rf_error("[%d] %s", i + 1, globalErrorMessage);
+    } else {
+      SET_VECTOR_ELT(result, i, geos_common_geometry_xptr(geometry));
+    }
+  }
+
+  GEOSGeoJSONReader_destroy_r(handle, reader);
+  UNPROTECT(1); // result
+  return result;
+
+#else
+  ERROR_OLD_LIBGEOS_BUILD("GEOSGeoJSONReader_create_r()", "3.10.0");
+#endif
+}
+
+SEXP geos_c_write_geojson(SEXP input, SEXP indent) {
+#if LIBGEOS_VERSION_COMPILE_INT >= LIBGEOS_VERSION_INT(3, 10, 0)
+  if (libgeos_version_int() < LIBGEOS_VERSION_INT(3, 10, 0)) {
+    ERROR_OLD_LIBGEOS("GEOSGeoJSONWriter_create_r()", "3.10.0");
+  }
+
+  R_xlen_t size = Rf_xlength(input);
+
+  int indent_int = INTEGER(indent)[0];
+
+  SEXP result = PROTECT(Rf_allocVector(STRSXP, size));
+
+  GEOS_INIT();
+  GEOSGeoJSONWriter* writer = GEOSGeoJSONWriter_create_r(handle);
+
+  SEXP item;
+  GEOSGeometry* geometry;
+  for (R_xlen_t i = 0; i < size; i++) {
+    item = VECTOR_ELT(input, i);
+
+    if (item == R_NilValue) {
+      SET_STRING_ELT(result, i, NA_STRING);
+      continue;
+    }
+
+    geometry = (GEOSGeometry*) R_ExternalPtrAddr(item);
+    if (geometry == NULL) {
+      GEOSGeoJSONWriter_destroy_r(handle, writer);
+      GEOS_CHECK_GEOMETRY(geometry, i);
+    }
+
+    char* output = GEOSGeoJSONWriter_writeGeometry_r(handle, writer, geometry, indent_int);
+    if (output == NULL) {
+      // don't know how to make this occur
+      GEOSGeoJSONWriter_destroy_r(handle, writer); // # nocov
+      Rf_error("[%d] %s", i + 1, globalErrorMessage); // # nocov
+    }
+
+    SET_STRING_ELT(result, i, Rf_mkChar(output));
+    GEOSFree_r(handle, output);
+  }
+
+  GEOSGeoJSONWriter_destroy_r(handle, writer);
+  UNPROTECT(1); // result
+  return result;
+
+#else
+  ERROR_OLD_LIBGEOS_BUILD("GEOSGeoJSONWriter_create_r()", "3.10.0");
+#endif
+}
+
 SEXP geos_c_read_wkb(SEXP input) {
   R_xlen_t size = Rf_xlength(input);
   SEXP result = PROTECT(Rf_allocVector(VECSXP, size));
@@ -115,7 +205,7 @@ SEXP geos_c_read_wkb(SEXP input) {
   return result;
 }
 
-SEXP geos_c_write_wkb(SEXP input, SEXP includeZ, SEXP includeSRID, SEXP endian) {
+SEXP geos_c_write_wkb(SEXP input, SEXP includeZ, SEXP includeSRID, SEXP endian, SEXP flavor) {
   R_xlen_t size = Rf_xlength(input);
   SEXP result = PROTECT(Rf_allocVector(VECSXP, size));
 
@@ -135,6 +225,33 @@ SEXP geos_c_write_wkb(SEXP input, SEXP includeZ, SEXP includeSRID, SEXP endian)
     GEOSWKBWriter_setIncludeSRID_r(handle, writer, 0);
   }
 
+  int flavor_default = 1;
+  int flavor_int = INTEGER(flavor)[0];
+
+  // in GEOS 3.10.0 the constants required to specify ISO vs EWKB
+  // are swapped from those noted by the enum in the C API
+  if (libgeos_version_int() == LIBGEOS_VERSION_INT(3, 10, 0)) {
+    flavor_default = 2;
+
+    if (flavor_int == 2) {
+      flavor_int = 1;
+    } else if (flavor_int == 1) {
+      flavor_int = 2;
+    }
+  }
+
+  if (flavor_int != flavor_default) {
+#if LIBGEOS_VERSION_COMPILE_INT >= LIBGEOS_VERSION_INT(3, 10, 0)
+    if (libgeos_version_int() < LIBGEOS_VERSION_INT(3, 10, 0)) {
+      ERROR_OLD_LIBGEOS("GEOSWKBWriter_setFlavor_r()", "3.10.0");
+    }
+
+    GEOSWKBWriter_setFlavor_r(handle, writer, flavor_int);
+#else
+    ERROR_OLD_LIBGEOS_BUILD("GEOSWKBWriter_setFlavor_r()", "3.10.0");
+#endif
+  }
+
   SEXP item;
   GEOSGeometry* geometry;
   size_t itemSize;
@@ -167,7 +284,7 @@ SEXP geos_c_write_wkb(SEXP input, SEXP includeZ, SEXP includeSRID, SEXP endian)
   }
 
   GEOSWKBWriter_destroy_r(handle, writer);
-    UNPROTECT(1); // result
+  UNPROTECT(1); // result
   return result;
 }
 
@@ -206,7 +323,7 @@ SEXP geos_c_read_hex(SEXP input) {
   return result;
 }
 
-SEXP geos_c_write_hex(SEXP input, SEXP includeZ, SEXP includeSRID, SEXP endian) {
+SEXP geos_c_write_hex(SEXP input, SEXP includeZ, SEXP includeSRID, SEXP endian, SEXP flavor) {
   R_xlen_t size = Rf_xlength(input);
   SEXP result = PROTECT(Rf_allocVector(STRSXP, size));
 
@@ -226,6 +343,33 @@ SEXP geos_c_write_hex(SEXP input, SEXP includeZ, SEXP includeSRID, SEXP endian)
     GEOSWKBWriter_setIncludeSRID_r(handle, writer, 0);
   }
 
+  int flavor_default = 1;
+  int flavor_int = INTEGER(flavor)[0];
+
+  // in GEOS 3.10.0 the constants required to specify ISO vs EWKB
+  // are swapped from those noted by the enum in the C API
+  if (libgeos_version_int() == LIBGEOS_VERSION_INT(3, 10, 0)) {
+    flavor_default = 2;
+
+    if (flavor_int == 2) {
+      flavor_int = 1;
+    } else if (flavor_int == 1) {
+      flavor_int = 2;
+    }
+  }
+
+  if (flavor_int != flavor_default) {
+#if LIBGEOS_VERSION_COMPILE_INT >= LIBGEOS_VERSION_INT(3, 10, 0)
+    if (libgeos_version_int() < LIBGEOS_VERSION_INT(3, 10, 0)) {
+      ERROR_OLD_LIBGEOS("GEOSWKBWriter_setFlavor_r()", "3.10.0");
+    }
+
+    GEOSWKBWriter_setFlavor_r(handle, writer, flavor_int);
+#else
+    ERROR_OLD_LIBGEOS_BUILD("GEOSWKBWriter_setFlavor_r()", "3.10.0");
+#endif
+  }
+
   SEXP item;
   GEOSGeometry* geometry;
   size_t itemSize;
@@ -297,7 +441,7 @@ SEXP geos_c_write_xy(SEXP input) {
     }
   }
 
-  
+
   const char* names[] = {"x", "y", ""};
   SEXP result = PROTECT(Rf_mkNamed(VECSXP, names));
   SET_VECTOR_ELT(result, 0, resultX);
